"use strict";
/*******************************************************************************
 * Copyright (c) 2019 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v2.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v20.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const vscode = require("vscode");
const request = require("request-promise-native");
const CommandUtil_1 = require("./CommandUtil");
const Logger_1 = require("../Logger");
const MCUtil = require("../MCUtil");
const UserProjectCreator_1 = require("../microclimate/connection/UserProjectCreator");
const Endpoints_1 = require("../constants/Endpoints");
/**
 * @param create true for Create page, false for Import page
 */
function createProject(connection) {
    return __awaiter(this, void 0, void 0, function* () {
        if (connection == null) {
            const selected = yield CommandUtil_1.promptForConnection(true);
            if (selected == null) {
                Logger_1.default.d("User cancelled prompt for resource");
                // user cancelled
                return;
            }
            connection = selected;
        }
        try {
            const template = yield promptForTemplate(connection);
            if (template == null) {
                return;
            }
            const projectName = yield promptForProjectName(template);
            if (projectName == null) {
                return;
            }
            const response = yield UserProjectCreator_1.default.createProject(connection, template, projectName);
            if (!response) {
                // user cancelled
                return;
            }
            vscode.window.showInformationMessage(`Created project ${response.projectName} at ${response.projectPath}`);
        }
        catch (err) {
            const errMsg = "Error creating project from template: ";
            Logger_1.default.e(errMsg, err);
            vscode.window.showErrorMessage(errMsg + MCUtil.errToString(err));
        }
    });
}
exports.default = createProject;
function promptForTemplate(connection) {
    return __awaiter(this, void 0, void 0, function* () {
        const templatesUrl = Endpoints_1.default.resolveMCEndpoint(connection, Endpoints_1.MCEndpoints.TEMPLATES);
        const templates = yield request.get(templatesUrl, { json: true });
        const projectTypeQpis = templates.map((type) => {
            return Object.assign({}, type, { detail: type.language, extension: type.extension });
        });
        return vscode.window.showQuickPick(projectTypeQpis, {
            placeHolder: "Select the project type to create",
            // matchOnDescription: true,
            matchOnDetail: true,
        });
    });
}
function promptForProjectName(template) {
    return __awaiter(this, void 0, void 0, function* () {
        return yield vscode.window.showInputBox({
            placeHolder: `Enter a name for your new ${template.language} project`,
            validateInput: validateProjectName,
        });
    });
}
// const ILLEGAL_CHARS = [
// `"`, "/", "\\", "?", "%", "*", ":", "|", "<", ">",
// ];
function validateProjectName(projectName) {
    // const firstIllegalChar = [...projectName].find((c) => ILLEGAL_CHARS.includes(c));
    // if (firstIllegalChar != null) {
    //     return `Invalid project name "${projectName}". Project names may not contain "${firstIllegalChar}"`;
    // }
    const matches = /^[a-z0-9_.-]+$/.test(projectName);
    if (!matches) {
        return `Invalid project name "${projectName}". Project name can only contain numbers, lowercase letters, periods, hyphens, and underscores.`;
    }
    return undefined;
}
//# sourceMappingURL=CreateUserProjectCmd.js.map