"use strict";
/*******************************************************************************
 * Copyright (c) 2019 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v2.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v20.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const vscode = require("vscode");
const path = require("path");
const child_process = require("child_process");
const fs = require("fs");
const os = require("os");
const MCUtil = require("../../MCUtil");
const Logger_1 = require("../../Logger");
const BIN_DIR = "bin";
const INSTALLER_DIR = "installer";
const INSTALLER_EXECUTABLE = "installer";
const INSTALLER_EXECUTABLE_WIN = "installer.exe";
var InstallerCommands;
(function (InstallerCommands) {
    // INSTALL = "install",
    InstallerCommands["START"] = "start";
    InstallerCommands["STOP"] = "stop";
    InstallerCommands["STOP_ALL"] = "stop-all";
})(InstallerCommands || (InstallerCommands = {}));
const INSTALLER_COMMANDS = {
    start: { action: "start", userActionName: "Starting Codewind" },
    stop: { action: "stop", userActionName: "Stopping Codewind" },
    "stop-all": { action: "stop-all", userActionName: "Stopping Codewind and applications" },
};
const START_TIMEOUT = 60000;
var InstallerWrapper;
(function (InstallerWrapper) {
    /**
     * Returns the location of the executable as within the extension. It cannot be run from this location - see prepare()
     */
    function getInternalExecutable() {
        const platform = MCUtil.getOS();
        const executable = platform === "windows" ? INSTALLER_EXECUTABLE_WIN : INSTALLER_EXECUTABLE;
        if (platform === "windows") {
            throw new Error("The installer does not support Windows at this time.");
        }
        return path.join(global.__extRoot, BIN_DIR, INSTALLER_DIR, platform, executable);
    }
    let tmpDir;
    let executableLoc;
    // serves as a lock, only one operation at a time.
    let currentOperation;
    function prepare() {
        return __awaiter(this, void 0, void 0, function* () {
            // We have to copy the executable to somewhere this user has write permissions so it can write out ./installer-docker-compose
            tmpDir = os.tmpdir();
            const executable = getInternalExecutable();
            const executableBasename = path.basename(executable);
            const tmpExecutableLoc = path.join(tmpDir, executableBasename);
            Logger_1.default.d(`Copying ${executable} to ${tmpExecutableLoc}`);
            fs.copyFileSync(executable, path.join(tmpDir, executableBasename));
            Logger_1.default.d("Installer copy succeeded");
            return tmpExecutableLoc;
        });
    }
    function getUserActionName(cmd) {
        return INSTALLER_COMMANDS[cmd].userActionName;
    }
    function installerExec(cmd) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!executableLoc) {
                // do this once only
                executableLoc = yield prepare();
            }
            // tmpDir will now be set too
            else if (currentOperation != null) {
                vscode.window.showWarningMessage(`Already ${getUserActionName(cmd)}`);
                return;
            }
            const userMsg = getUserActionName(cmd) + "...";
            currentOperation = cmd;
            yield vscode.window.withProgress({
                cancellable: false,
                location: vscode.ProgressLocation.Notification,
                title: userMsg,
            }, (_progress) => {
                return new Promise((resolve, reject) => {
                    child_process.execFile(executableLoc, [cmd], {
                        cwd: tmpDir,
                        timeout: START_TIMEOUT,
                    }, (err, stdout, stderr) => __awaiter(this, void 0, void 0, function* () {
                        if (err) {
                            Logger_1.default.e("Error starting with installer", err);
                            Logger_1.default.e("Stdout:", stdout.toString());
                            Logger_1.default.e("Stderr:", stderr.toString());
                            currentOperation = undefined;
                            return reject(err);
                        }
                        Logger_1.default.i("Successfully started CW with installer");
                        if (cmd === InstallerCommands.START) {
                            // additional delay
                            yield new Promise((resolve2) => setTimeout(resolve2, 5000));
                        }
                        currentOperation = undefined;
                        resolve();
                        Logger_1.default.d("Finished starting Codewind");
                    }));
                });
            });
        });
    }
    function start() {
        return __awaiter(this, void 0, void 0, function* () {
            return installerExec(InstallerCommands.START);
        });
    }
    InstallerWrapper.start = start;
    function stop() {
        return __awaiter(this, void 0, void 0, function* () {
            return installerExec(InstallerCommands.STOP);
        });
    }
    InstallerWrapper.stop = stop;
    function stopAll() {
        return __awaiter(this, void 0, void 0, function* () {
            return installerExec(InstallerCommands.STOP_ALL);
        });
    }
    InstallerWrapper.stopAll = stopAll;
})(InstallerWrapper || (InstallerWrapper = {}));
exports.default = InstallerWrapper;
//# sourceMappingURL=InstallerWrapper.js.map