"use strict";
/*******************************************************************************
 * Copyright (c) 2019 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v2.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v20.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const vscode = require("vscode");
const path = require("path");
const request = require("request-promise-native");
const Logger_1 = require("../../Logger");
const Endpoints_1 = require("../../constants/Endpoints");
const SocketEvents_1 = require("./SocketEvents");
/**
 * Functions to create or import new user projects into Codewind
 */
var UserProjectCreator;
(function (UserProjectCreator) {
    function createProject(connection, template, projectName) {
        return __awaiter(this, void 0, void 0, function* () {
            // right now projects must be created under the microclimate-workspace so users can't cohoose the parentDir
            const parentDirUri = connection.workspacePath;
            // abs path on user system under which the project will be created
            const userParentDir = parentDirUri.fsPath;
            // caller must handle errors
            const creationRes = yield requestCreate(connection, template, projectName, userParentDir);
            if (creationRes.status !== SocketEvents_1.default.STATUS_SUCCESS) {
                // failed
                const failedResult = creationRes.result;
                throw new Error(failedResult.error);
            }
            const result = creationRes.result;
            // create succeeded, now we bind
            // const bindRes = await requestBind(connection, projectName, creationRes.projectPath, result.language, result.buildType);
            yield requestBind(connection, projectName, creationRes.projectPath, result.language, result.projectType);
            return { projectName, projectPath: creationRes.projectPath };
        });
    }
    UserProjectCreator.createProject = createProject;
    function validateAndBind(connection, pathToBindUri) {
        return __awaiter(this, void 0, void 0, function* () {
            const pathToBind = pathToBindUri.fsPath;
            Logger_1.default.i("Binding to", pathToBind);
            const projectName = path.basename(pathToBind);
            const validateRes = yield preBindValidate(connection, pathToBind);
            if (validateRes.status !== SocketEvents_1.default.STATUS_SUCCESS) {
                // failed
                const failedResult = validateRes.result;
                throw new Error(failedResult.error);
            }
            const result = validateRes.result;
            yield requestBind(connection, projectName, pathToBind, result.language, result.projectType);
            return { projectName, projectPath: pathToBind };
        });
    }
    UserProjectCreator.validateAndBind = validateAndBind;
    function preBindValidate(connection, pathToBind) {
        return __awaiter(this, void 0, void 0, function* () {
            const validateResponse = yield request.post(Endpoints_1.default.resolveMCEndpoint(connection, Endpoints_1.MCEndpoints.PREBIND_VALIDATE), {
                json: true,
                body: {
                    projectPath: pathToBind,
                }
            });
            Logger_1.default.d("validate response", validateResponse);
            return validateResponse;
        });
    }
    function promptForDir(btnLabel, defaultUri) {
        return __awaiter(this, void 0, void 0, function* () {
            // if (!defaultUri && vscode.workspace.workspaceFolders != null) {
            //     defaultUri = vscode.workspace.workspaceFolders[0].uri;
            // }
            const selectedDirs = yield vscode.window.showOpenDialog({
                canSelectFiles: false,
                canSelectFolders: true,
                canSelectMany: false,
                openLabel: btnLabel,
                defaultUri
            });
            if (selectedDirs == null) {
                return;
            }
            // canSelectMany is false
            return selectedDirs[0];
        });
    }
    UserProjectCreator.promptForDir = promptForDir;
    function requestCreate(connection, projectTypeSelected, projectName, projectLocation) {
        return __awaiter(this, void 0, void 0, function* () {
            const payload = {
                projectName: projectName,
                templateID: projectTypeSelected.extension,
                parentPath: projectLocation,
            };
            Logger_1.default.d("Creation request", payload);
            const creationRes = yield request.post(Endpoints_1.default.resolveMCEndpoint(connection, Endpoints_1.MCEndpoints.PROJECTS), {
                json: true,
                body: payload,
            });
            Logger_1.default.i("Creation response", creationRes);
            return creationRes;
        });
    }
    UserProjectCreator.requestCreate = requestCreate;
    function requestBind(connection, projectName, dirToBind, language, projectType) {
        return __awaiter(this, void 0, void 0, function* () {
            const bindReq = {
                name: projectName,
                language,
                projectType,
                path: dirToBind,
            };
            Logger_1.default.d("Bind request", bindReq);
            const bindEndpoint = Endpoints_1.default.resolveMCEndpoint(connection, Endpoints_1.MCEndpoints.BIND);
            const bindRes = yield request.post(bindEndpoint, {
                json: true,
                body: bindReq,
            });
            Logger_1.default.d("Bind response", bindRes);
            // return bindRes;
        });
    }
})(UserProjectCreator || (UserProjectCreator = {}));
exports.default = UserProjectCreator;
//# sourceMappingURL=UserProjectCreator.js.map